#!/usr/bin/env python3

# Copyright (c) 2021 - 2021 TomTom N.V.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
import logging

from . import report, generate


LOGGER = logging.getLogger(__name__)


@click.group()
@click.option("--verbose", is_flag=True, help="Enable verbose output")
def main(verbose):
    logging.basicConfig(
        level=logging.DEBUG if verbose else logging.INFO,
        format="%(levelname)s: %(message)s",
    )


for submodule in (
    report,
    generate,
):
    for subcmd in dir(submodule):
        if subcmd.startswith("_"):
            continue
        cmd = getattr(submodule, subcmd)
        if not isinstance(cmd, click.Command):
            continue
        main.add_command(cmd)



#!/usr/bin/env python3

# Copyright (c) 2021 - 2021 TomTom N.V.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
import json
import logging
import os

from llvm_diagnostics import parser

LOGGER = logging.getLogger(__name__)


def get_severity_for_level(level):
    if level == "error":
        return "HIGH"

    if level == "warning":
        return "MEDIUM"

    return "LOW"


def retrieve_annotations_from_file(path, workspace):
    _annotations = []
    for diag_msg in parser.diagnostics_messages_from_file(path):
        _data = json.loads(diag_msg.to_json())
        _filepath = (
            _data["filepath"]
            if not workspace
            else _data["filepath"].replace(workspace, "").lstrip(os.path.sep)
        )
        _annotations.append(
            {
                "path": _filepath,
                "line": _data["line"],
                "message": _data["message"],
                "severity": get_severity_for_level(_data["level"]),
            }
        )

    return _annotations


@click.command()
@click.option(
    "--id",
    required=True,
    help="Unique identifier for the report",
)
@click.option(
    "--title",
    required=True,
    help="Humand readable title for the Code Insight report",
)
@click.option(
    "--details",
    help="Additional details to share withing the Code Insight report",
)
@click.option(
    "--reporter",
    help="Reference to the reporter of the Code Insight Report",
)
@click.option(
    "--link",
    help="Link towards an external report",
)
@click.option(
    "--logo-url",
    help="Link towards an image to be shown in the Code Insight report",
)
@click.option(
    "--workspace",
    help="Absolute path towards the root of the repository. This will be stripped from the file paths in the LLVM logging.",
)
@click.option(
    "--llvm-logging",
    required=True,
    help="Path pointing to logging file containing llvm diagnostics messages",
)
@click.option(
    "--output",
    required=True,
    type=click.File("w", encoding="UTF-8", lazy=True, atomic=True),
    help="Path towards the output file",
)
def generate(
    id,
    title,
    details,
    reporter,
    link,
    logo_url,
    workspace,
    llvm_logging,
    output,
):
    LOGGER.info("BitBucket Code Insight Generator")

    if not os.path.exists(llvm_logging):
        LOGGER.error("Specified input file does not exist!")
        return 1

    _report = {"id": id, "report": {}}

    if title:
        _report["report"]["title"] = title
    if details:
        _report["report"]["details"] = details
    if reporter:
        _report["report"]["reporter"] = reporter
    if link:
        _report["report"]["link"] = link
    if logo_url:
        _report["report"]["logo-url"] = logo_url

    _annotations = retrieve_annotations_from_file(llvm_logging, workspace)
    if _annotations:
        _report["annotations"] = _annotations

    _failure = len(_annotations) > 0

    if _failure:
        _report["report"]["result"] = "FAIL"

    LOGGER.debug(f"Generating Report: {json.dumps(_report, indent=4, sort_keys=True)}")

    output.write(json.dumps(_report))
    LOGGER.info(f"Done...")

    return 0



# Copyright (c) 2021 - 2021 TomTom N.V.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

from setuptools import setup

with open('README.rst') as file:
    long_description = file.read()

setup(
    name='bitbucket-code-insight-reporter',
    description='Allows for parsing LLVM Diagnostics output and create BB code insights accordingly',
    long_description=long_description,
    download_url='https://github.com/KevinDeJong-TomTom/bitbucket-code-insight-reporter',
    url='https://github.com/KevinDeJong-TomTom/bitbucket-code-insight-reporter',
    author='Kevin de Jong',
    author_email='KevinDeJong@tomtom.com',
    keywords='diagnostics codeinsights insights bitbucket llvm',
    license='Apache License 2.0',
    license_files='LICENSE.txt',
    packages=(
        'bitbucket_code_insight_reporter',
    ),
    python_requires='>=3.5',
    setup_requires=(
        'setuptools_scm',
        'setuptools_scm_git_archive',
    ),
    install_requires=(
        'Click>=7,<8',
        'llvm_diagnostics>=0<1',
    ),
    entry_points={
        'console_scripts': [
            'code_insight_reporter=bitbucket_code_insight_reporter:main',
        ],
    },
    use_scm_version={"relative_to": __file__},
    zip_safe=True,
)



#!/usr/bin/env python3

# Copyright (c) 2021 - 2021 TomTom N.V.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
import json
import logging

from .bitbucket import Bitbucket


@click.command()
@click.option(
    "--bitbucket-server",
    required=True,
    help="URL for the BitBucket server",
)
@click.option(
    "--username",
    required=True,
    prompt=True,
    help="Username associated with BitBucket",
)
@click.option(
    "--password",
    required=True,
    prompt=True,
    hide_input=True,
    help="Password associated with BitBucket",
)
@click.option(
    "--bitbucket-project",
    required=True,
    help="BitBucket project name",
)
@click.option(
    "--repository-slug",
    required=True,
    help="BitBucket repository slug name",
)
@click.option(
    "--commit-hash",
    required=True,
    help="Commit Hash to associate the Code Insights Report with",
)
@click.option(
    "--report-file",
    type=click.File("r", encoding="UTF-8", lazy=True),
    required=True,
    help="Code Insights Report identifier",
)
def report(
    bitbucket_server,
    username,
    password,
    bitbucket_project,
    repository_slug,
    commit_hash,
    report_file,
):
    logging.info("BitBucket Code Insight Reporter")

    _report = json.load(report_file)
    _report_id = _report["id"]

    bitbucket = Bitbucket(
        url=bitbucket_server,
        username=username,
        password=password,
    )

    try:
        bitbucket.delete_code_insights_report(
            project_key=bitbucket_project,
            repository_slug=repository_slug,
            commit_id=commit_hash,
            report_key=_report_id,
        )
    except:
        pass

    logging.debug(
        f"""\
Project: {bitbucket_project}
Repository: {repository_slug}
Commit Hash: {commit_hash}
Report: {json.dumps(_report["report"], indent=4, sort_keys=True)}"""
    )

    try:
        bitbucket.create_code_insights_report(
            project_key=bitbucket_project,
            repository_slug=repository_slug,
            commit_id=commit_hash,
            report_key=_report_id,
            report_title=_report["report"]["title"],
            **_report["report"],
        )
    except:
        print("ERR - Failed to create new Code Insight Report")
        return 1

    _annotations = _report["annotations"]
    if not _annotations:
        return 0

    logging.debug(
        f"""\
Project: {bitbucket_project}
Repository: {repository_slug}
Commit Hash: {commit_hash}
Annotations: {json.dumps(_annotations, indent=4, sort_keys=True)}"""
    )

    bitbucket.add_code_insights_annotations_to_report(
        project_key=bitbucket_project,
        repository_slug=repository_slug,
        commit_id=commit_hash,
        report_key=_report_id,
        annotations=_annotations,
    )

    logging.info("Done...")

    return 0
